from abc import ABC, abstractmethod, abstractproperty
from typing import Dict, Optional

class IDexProvider(ABC):
    """DEX 提供者接口"""
    
    @abstractmethod
    def client(self):
        """获取底层客户端实例"""
        pass
    
    @abstractmethod
    def get_quote(self, chain_id: str, from_token: str, 
                  to_token: str, amount: str, **kwargs) -> Dict:
        """获取报价"""
        pass
    
    @abstractmethod
    def swap(self, chain_id: str, from_token: str,to_token: str, amount: str, recipient_address: Optional[str] = None,  slippage: str = "0.03", **kwargs) -> str:
        """执行兑换"""
        pass
    
    @abstractmethod
    def check_and_approve(self, chain_id: str, token_address: str, 
                         owner_address: str, amount: int) -> Optional[str]:
        """检查并处理代币授权"""
        pass


from typing import Dict, Optional
import base58
from solders.transaction import VersionedTransaction, Transaction
from solders.message import MessageV0
from solders.keypair import Keypair
from solders.pubkey import Pubkey
from solana.rpc.api import Client
from spl.token.client import Token
from spl.token.constants import TOKEN_PROGRAM_ID
from ...core.interfaces import IDexProvider
from ...config.settings import WALLET_CONFIG, NATIVE_TOKENS, WEB3_CONFIG
from ...utils.logger import get_logger
from .client import OKXClient

logger = get_logger(__name__)

class OKXSolanaProvider(IDexProvider):
    """OKX Solana 提供商实现"""

    SOLANA_CHAIN_ID = "501"
    MAX_RETRIES = 3

    def __init__(self):
        self._client = OKXClient()
        self.wallet_config = WALLET_CONFIG.get("solana", WALLET_CONFIG["default"])
        self.solana_client = Client(WEB3_CONFIG['providers'][self.SOLANA_CHAIN_ID])

    @property
    def client(self):
        return self._client

    def _get_token_decimals(self, token_address: str) -> int:
        """获取 token 精度"""
        try:
            chain_id = self.SOLANA_CHAIN_ID
            # 处理原生 token
            if token_address.lower() == "11111111111111111111111111111111":
                if chain_id not in NATIVE_TOKENS:
                    raise ValueError(f"Unsupported chain ID: {chain_id}")
                return NATIVE_TOKENS[chain_id]["decimals"]

            # 处理 SPL token
            token = Token(
                conn=self.solana_client,
                pubkey=Pubkey.from_string(token_address),
                program_id=TOKEN_PROGRAM_ID,
                payer=None
            )
            mint_info = token.get_mint_info()
            return mint_info.decimals

        except Exception as e:
            logger.error(f"Failed to get token decimals for {token_address}: {str(e)}")
            raise

    def _convert_amount(self, amount: str, token_address: str) -> str:
        """将金额转换为链上小数"""
        try:
            value = float(amount)
            if value <= 0:
                raise ValueError("Amount must be greater than 0")
            decimals = self._get_token_decimals(token_address)
            return str(int(value * (10 ** decimals)))
        except (ValueError, TypeError) as e:
            logger.error(f"Amount conversion error: {str(e)}")
            raise ValueError("Invalid amount format")

    def get_quote(self, chain_id: str, from_token: str, to_token: str, amount: str, **kwargs) -> Dict:
        """获取兑换参数"""
        try:
            if chain_id != self.SOLANA_CHAIN_ID:
                raise ValueError(f"Invalid chain ID for Solana: {chain_id}")

            raw_amount = self._convert_amount(amount, from_token)
            params = {
                "chainId": chain_id,
                "fromTokenAddress": from_token,
                "toTokenAddress": to_token,
                "amount": raw_amount,
                **kwargs
            }

            quote_result = self.client.get_quote(params)
            logger.info(f"Got quote for {amount} from {from_token} to {to_token}")
            return quote_result

        except Exception as e:
            logger.error(f"Failed to get quote: {str(e)}")
            raise

    def check_and_approve(self, chain_id: str, token_address: str, owner_address: str, amount: int) -> Optional[str]:
        """Solana 不需要授权"""
        return None

    def swap(self, chain_id: str, from_token: str, to_token: str, amount: str,
             recipient_address: Optional[str] = None,slippage: str = "0.03", **kwargs) -> str:
        """在 Solana 上执行兑换交易"""
        try:
            if chain_id != self.SOLANA_CHAIN_ID:
                raise ValueError(f"Invalid chain ID for Solana: {chain_id}")

            # 检查是否为原生 SOL 并验证余额
            if from_token.lower() == "11111111111111111111111111111111":
                balance = self.solana_client.get_balance(
                    Pubkey.from_string(self.wallet_config["address"])
                ).value
                raw_amount = int(self._convert_amount(amount, from_token))
                if balance < raw_amount:
                    raise ValueError(
                        f"SOL 余额不足。需要: {amount}, 可用: {balance / 1e9}"
                    )
                logger.info(f"SOL 余额检查通过。余额: {balance / 1e9} SOL")

            # 获取兑换数据
            params = {
                "chainId": chain_id,
                "fromTokenAddress": from_token,
                "toTokenAddress": to_token,
                "amount": self._convert_amount(amount, from_token),
                "userWalletAddress": self.wallet_config["address"],
                "slippage": slippage,
            }

            if recipient_address:
                params["swapReceiverAddress"] = recipient_address

            swap_data = self.client.get_swap(params)
            logger.info(f"获取兑换数据 {amount} 从 {from_token} 到 {to_token}")

            if "data" not in swap_data or not swap_data["data"]:
                raise ValueError("收到无效的兑换数据")

            tx_data = swap_data["data"][0]["tx"]["data"]

            # 解码交易数据
            try:
                decoded_tx = base58.b58decode(tx_data)
            except Exception as e:
                logger.error(f"解码交易数据失败: {e}")
                raise

            # 获取密钥对
            keypair = Keypair.from_bytes(base58.b58decode(self.wallet_config["private_key"]))
            logger.info(f"使用钱包地址: {self.wallet_config['address']}")

            # 获取最新的区块哈希
            latest_blockhash = self.solana_client.get_latest_blockhash().value.blockhash
            logger.info(f"获取最新区块哈希: {latest_blockhash}")

            try:
                # 尝试创建并签名版本化交易
                tx = VersionedTransaction.from_bytes(decoded_tx)
                
                # 使用更新的区块哈希创建新消息
                message = MessageV0(
                    header=tx.message.header,
                    account_keys=tx.message.account_keys,
                    recent_blockhash=latest_blockhash,
                    instructions=tx.message.instructions,
                    address_table_lookups=tx.message.address_table_lookups
                )
                
                # 创建并签名交易
                signed_tx = VersionedTransaction(message, [keypair])
                try:
                    serialized_tx = bytes(signed_tx)  # 序列化已签名的交易
                    signature = self.solana_client.send_raw_transaction(serialized_tx).value
                    tx_hash = str(signature)

                    logger.info(f"兑换交易已发送: {tx_hash}")
                    return tx_hash

                except Exception as e:
                    logger.error(f"发送交易失败: {e}")
                    raise

            except Exception as e:
                logger.error(f"创建和签名交易失败: {e}")
                raise

        except Exception as e:
            logger.error(f"执行兑换失败: {str(e)}")
            raise


import os
from typing import Dict
from dotenv import load_dotenv

# 加载环境变量
load_dotenv()

# API配置
API_CONFIG = {
    "OKX": {
        "base_url": "https://www.okx.com/api/v5/dex/aggregator",
        "api_key": os.getenv("OKX_API_KEY"),
        "secret_key": os.getenv("OKX_SECRET_KEY"),
        "passphrase": os.getenv("OKX_PASSPHRASE"),
    }
}

# Web3配置
WEB3_CONFIG = {
    "providers": {
        "1": os.getenv("ETH_RPC_URL"),
        "56": os.getenv("BSC_RPC_URL"),
        "137": os.getenv("POLYGON_RPC_URL"),
        "42161": os.getenv("ARBITRUM_RPC_URL"),
        "10": os.getenv("OPTIMISM_RPC_URL"),
        "43114": os.getenv("AVALANCHE_RPC_URL"),
        "501": os.getenv("SOLANA_RPC_URL", "https://api.mainnet-beta.solana.com")
    }
}

# 钱包配置
WALLET_CONFIG = {
    "default": {
        "address": os.getenv("DEFAULT_WALLET_ADDRESS"),
        "private_key": os.getenv("DEFAULT_WALLET_PRIVATE_KEY")
    },
    "solana": {
        "address": os.getenv("SOLANA_WALLET_ADDRESS"),
        "private_key": os.getenv("SOLANA_WALLET_PRIVATE_KEY")
    }
}

# 如果设置了其他钱包的环境变量，添加到配置中
if os.getenv("WALLET2_ADDRESS") and os.getenv("WALLET2_PRIVATE_KEY"):
    WALLET_CONFIG["wallet2"] = {
        "address": os.getenv("WALLET2_ADDRESS"),
        "private_key": os.getenv("WALLET2_PRIVATE_KEY")
    }

# 原生代币配置
NATIVE_TOKENS = {
    "1": {
        "symbol": "ETH",
        "decimals": 18,
        "name": "Ethereum"
    },
    "56": {
        "symbol": "BNB",
        "decimals": 18,
        "name": "BNB Chain"
    },
    "137": {
        "symbol": "MATIC",
        "decimals": 18,
        "name": "Polygon"
    },
    "42161": {
        "symbol": "ETH",
        "decimals": 18,
        "name": "Arbitrum"
    },
    "10": {
        "symbol": "ETH",
        "decimals": 18,
        "name": "Optimism"
    },
    "43114": {
        "symbol": "AVAX",
        "decimals": 18,
        "name": "Avalanche"
    },
    "501": {  # Solana
        "symbol": "SOL",
        "decimals": 9,
        "name": "Solana"
    }
}

# 日志配置
LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "standard": {
            "format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"
        },
    },
    "handlers": {
        "default": {
            "level": os.getenv("LOG_LEVEL", "INFO"),
            "formatter": "standard",
            "class": "logging.StreamHandler",
        },
    },
    "loggers": {
        "": {
            "handlers": ["default"],
            "level": os.getenv("LOG_LEVEL", "INFO"),
            "propagate": True
        },
    }
} 


from web3 import Web3, HTTPProvider
from typing import Dict, Optional, Union, List
from dex_aggregator.config.settings import WEB3_CONFIG
from dex_aggregator.core.exceptions import ConfigError
from web3.exceptions import ContractLogicError
from dex_aggregator.utils.abi_helper import ABIHelper
from dex_aggregator.utils.logger import get_logger
from decimal import Decimal
from web3.middleware import ExtraDataToPOAMiddleware

logger = get_logger(__name__)


class Web3Helper:
    _instances: Dict[str, 'Web3Helper'] = {}
    
    def __init__(self, chain_id: str):
        if chain_id not in WEB3_CONFIG["providers"]:
            raise ConfigError(f"Chain ID {chain_id} not supported")
        
        self.chain_id = chain_id
        self.web3 = Web3(HTTPProvider(WEB3_CONFIG["providers"][chain_id]))
        
        # 为 BSC 等 POA 链添加中间件
        if chain_id in ["56", "97"]:  # BSC 主网和测试网
            self.web3.middleware_onion.inject(ExtraDataToPOAMiddleware, layer=0)
        
        self.abi_helper = ABIHelper.get_instance()

    @classmethod
    def get_instance(cls, chain_id: str) -> 'Web3Helper':
        if chain_id not in cls._instances:
            cls._instances[chain_id] = cls(chain_id)
        return cls._instances[chain_id]

    def get_allowance(self, token_address: str, owner_address: str, spender_address: str) -> int:
        """查询代币授权额度"""
        contract = self.web3.eth.contract(
            address=self.web3.to_checksum_address(token_address),
            abi=self.abi_helper.get_abi('erc20')
        )
        return contract.functions.allowance(
            self.web3.to_checksum_address(owner_address),
            self.web3.to_checksum_address(spender_address)
        ).call()

    def send_transaction(self, transaction: Dict, private_key: str) -> str:
        """发送交易"""
        signed_txn = self.web3.eth.account.sign_transaction(
            transaction,
            private_key
        )
        tx_hash = self.web3.eth.send_raw_transaction(
            signed_txn.raw_transaction
        )
        return self.web3.to_hex(tx_hash)

    def get_token_info(self, token_address: str) -> Dict:
        """
        获取代币基本信息
        
        Args:
            token_address: 代币合约地址
            
        Returns:
            Dict: {
                'name': str,
                'symbol': str,
                'decimals': int,
                'total_supply': int
            }
        """
        try:
            contract = self.web3.eth.contract(
                address=self.web3.to_checksum_address(token_address),
                abi=self.abi_helper.get_abi('erc20')
            )
            
            return {
                'name': contract.functions.name().call(),
                'symbol': contract.functions.symbol().call(),
                'decimals': contract.functions.decimals().call(),
                'total_supply': contract.functions.totalSupply().call()
            }
        except ContractLogicError as e:
            logger.error(f"Failed to get token info for {token_address}: {str(e)}")
            raise

    def get_token_decimals(self, token_address: str) -> int:
        """获取代币精度
        Args:
            token_address: 代币合约地址
        Returns:
            int: 代币精度
        """
        try:
            contract = self.web3.eth.contract(
                address=self.web3.to_checksum_address(token_address),
                abi=self.abi_helper.get_abi('erc20')
            )
            return contract.functions.decimals().call()
        except ContractLogicError as e:
            logger.error(f"Failed to get decimals for token {token_address}: {str(e)}")
            raise


    def get_token_balance(self, token_address: str, wallet_address: str, abi: list) -> int:
        """
        获取代币余额
        
        Args:
            token_address: 代币合约地址
            wallet_address: 钱包地址
            abi: 代币合约ABI
            
        Returns:
            int: 代币余额(原始值，未经精度转换)
        """
        try:
            contract = self.web3.eth.contract(
                address=self.web3.to_checksum_address(token_address),
                abi=abi
            )
            return contract.functions.balanceOf(
                self.web3.to_checksum_address(wallet_address)
            ).call()
        except ContractLogicError as e:
            logger.error(f"Failed to get balance for token {token_address}, wallet {wallet_address}: {str(e)}")
            raise

    def format_token_amount(self, amount: int, decimals: int) -> str:
        """
        格式化代币金额（从原始值转换为带精度的字符串）
        
        Args:
            amount: 原始金额
            decimals: 代币精度
            
        Returns:
            str: 格式化后的金额
        """
        return str(amount / (10 ** decimals))

    def parse_token_amount(self, amount: str, decimals: int) -> int:
        """
        将带小数点的金额转换为链上精度
        
        Args:
            amount: 带小数点的金额字符串
            decimals: 代币精度
            
        Returns:
            int: 转换后的金额
        """
        try:
            # 移除字符串中的所有空格
            amount = amount.strip()
            # 将字符串转换为浮点数
            float_amount = float(amount)
            # 转换为链上精度
            raw_amount = int(float_amount * (10 ** decimals))
            return raw_amount
        except ValueError as e:
            logger.error(f"Failed to parse amount {amount}: {str(e)}")
            raise ValueError(f"Invalid amount format: {amount}")

    def is_contract(self, address: str) -> bool:
        """
        检查地址是否为合约地址
        
        Args:
            address: 以太坊地址
            
        Returns:
            bool: 是否为合约地址
        """
        code = self.web3.eth.get_code(self.web3.to_checksum_address(address))
        return code != b''

    def get_native_balance(self, address: str) -> int:
        """
        获取原生代币余额（如ETH、BNB等）
        
        Args:
            address: 钱包地址
            
        Returns:
            int: 原生代币余额(单位: Wei)
        """
        return self.web3.eth.get_balance(self.web3.to_checksum_address(address))

    def get_transaction(self, tx_hash: str) -> Dict:
        """
        获取交易详情
        
        Args:
            tx_hash: 交易哈希
            
        Returns:
            Dict: 交易详情
        """
        try:
            tx = self.web3.eth.get_transaction(tx_hash)
            return dict(tx)
        except Exception as e:
            logger.error(f"Failed to get transaction {tx_hash}: {str(e)}")
            raise

    def get_transaction_receipt(self, tx_hash: str) -> Optional[Dict]:
        """
        获取交易收据
        
        Args:
            tx_hash: 交易哈希
            
        Returns:
            Optional[Dict]: 交易收据，如果交易未确认则返回None
        """
        try:
            receipt = self.web3.eth.get_transaction_receipt(tx_hash)
            return dict(receipt) if receipt else None
        except Exception as e:
            logger.error(f"Failed to get transaction receipt {tx_hash}: {str(e)}")
            raise

    def wait_for_transaction(self, tx_hash: str, timeout: int = 180, poll_interval: float = 0.1) -> Dict:
        """
        等待交易确认
        
        Args:
            tx_hash: 交易哈希
            timeout: 超时时间（秒）
            poll_interval: 轮询间隔（秒）
            
        Returns:
            Dict: 交易收据
        """
        try:
            receipt = self.web3.eth.wait_for_transaction_receipt(
                tx_hash, timeout=timeout, poll_latency=poll_interval
            )
            return dict(receipt)
        except Exception as e:
            logger.error(f"Failed to wait for transaction {tx_hash}: {str(e)}")
            raise

    def estimate_gas(self, transaction: Dict) -> int:
        """
        估算交易gas用量
        
        Args:
            transaction: 交易参数
            
        Returns:
            int: 预估的gas用量
        """
        try:
            return self.web3.eth.estimate_gas(transaction)
        except Exception as e:
            logger.error(f"Failed to estimate gas: {str(e)}")
            raise

    def get_block(self, block_identifier: Union[str, int]) -> Dict:
        """
        获取区块信息
        
        Args:
            block_identifier: 区块号或区块哈希
            
        Returns:
            Dict: 区块信息
        """
        try:
            block = self.web3.eth.get_block(block_identifier)
            return dict(block)
        except Exception as e:
            logger.error(f"Failed to get block {block_identifier}: {str(e)}")
            raise

    def get_logs(self, from_block: int, to_block: int, address: Optional[str] = None, 
                 topics: Optional[List] = None) -> List[Dict]:
        """
        获取事件日志
        
        Args:
            from_block: 起始区块
            to_block: 结束区块
            address: 合约地址（可选）
            topics: 主题过滤器（可选）
            
        Returns:
            List[Dict]: 事件日志列表
        """
        try:
            filter_params = {
                'fromBlock': from_block,
                'toBlock': to_block
            }
            if address:
                filter_params['address'] = self.web3.to_checksum_address(address)
            if topics:
                filter_params['topics'] = topics

            logs = self.web3.eth.get_logs(filter_params)
            return [dict(log) for log in logs]
        except Exception as e:
            logger.error(f"Failed to get logs: {str(e)}")
            raise

    def get_transaction_count(self, address: str, block_identifier: str = 'latest') -> int:
        """
        获取账户的nonce值
        
        Args:
            address: 账户地址
            block_identifier: 区块标识符
            
        Returns:
            int: nonce值
        """
        try:
            return self.web3.eth.get_transaction_count(
                self.web3.to_checksum_address(address),
                block_identifier
            )
        except Exception as e:
            logger.error(f"Failed to get transaction count for {address}: {str(e)}")
            raise

    def get_gas_price(self) -> int:
        """
        获取当前gas价格
        
        Returns:
            int: gas价格（wei）
        """
        try:
            return self.web3.eth.gas_price
        except Exception as e:
            logger.error(f"Failed to get gas price: {str(e)}")
            raise

    def format_amount(self, amount: Union[int, str], decimals: int) -> str:
        """
        格式化代币金额（从wei转换为带小数点的字符串）
        
        Args:
            amount: 原始金额（wei）
            decimals: 代币精度
            
        Returns:
            str: 格式化后的金额
        """
        try:
            amount_decimal = Decimal(str(amount)) / Decimal(str(10 ** decimals))
            return f"{amount_decimal:.{decimals}f}".rstrip('0').rstrip('.')
        except Exception as e:
            logger.error(f"Failed to format amount {amount}: {str(e)}")
            raise

    def is_valid_address(self, address: str) -> bool:
        """
        检查地址是否有效
        
        Args:
            address: 以太坊地址
            
        Returns:
            bool: 是否为有效地址
        """
        try:
            return self.web3.is_address(address)
        except Exception as e:
            logger.error(f"Failed to validate address {address}: {str(e)}")
            return False 


from typing import Dict, Type
from .interfaces import IDexProvider
from ..providers.okx.provider import OKXProvider
from ..providers.okx.solana_provider import OKXSolanaProvider
from ..providers.uniswap.provider import UniswapProvider

class DexFactory:
    """DEX 工厂类"""
    
    _providers: Dict[str, Type[IDexProvider]] = {
        "okx": OKXProvider,
        "okx_solana": OKXSolanaProvider,
        "uniswap": UniswapProvider
    }
    
    @classmethod
    def create_provider(cls, provider_name: str) -> IDexProvider:
        """创建 DEX 提供者实例"""
        if provider_name not in cls._providers:
            raise ValueError(f"Unsupported provider: {provider_name}")
        return cls._providers[provider_name]() 


import logging
import logging.config
from typing import Optional
from dex_aggregator.config.settings import LOGGING_CONFIG

# 配置日志
logging.config.dictConfig(LOGGING_CONFIG)

def get_logger(name: Optional[str] = None) -> logging.Logger:
    """
    获取logger实例
    
    Args:
        name: 日志名称，通常使用 __name__
        
    Returns:
        logging.Logger: 配置好的logger实例
    """
    return logging.getLogger(name)

class LoggerMixin:
    """
    Logger Mixin类，可以被其他类继承以获得日志功能
    
    Example:
        class MyClass(LoggerMixin):
            def __init__(self):
                self.logger = self.get_logger()
    """
    
    @property
    def logger(self) -> logging.Logger:
        """
        获取当前类的logger
        
        Returns:
            logging.Logger: 当前类的logger实例
        """
        if not hasattr(self, '_logger'):
            self._logger = get_logger(self.__class__.__name__)
        return self._logger

def log_error(logger: logging.Logger):
    """
    错误日志装饰器
    
    Args:
        logger: Logger实例
        
    Example:
        @log_error(logger)
        def my_function():
            # 函数代码
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                logger.error(
                    f"Error in {func.__name__}: {str(e)}",
                    exc_info=True
                )
                raise
        return wrapper
    return decorator 


import json
import os
from typing import Dict
from dex_aggregator.core.exceptions import ConfigError
from dex_aggregator.utils.logger import get_logger

logger = get_logger(__name__)

class ABIHelper:
    _instance = None
    _abis: Dict[str, list] = {}
    
    def __init__(self):
        self._load_abis()
    
    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = cls()
        return cls._instance
    
    def _load_abis(self):
        """加载所有ABI文件"""
        current_dir = os.path.dirname(os.path.dirname(__file__))
        abi_dir = os.path.join(current_dir, 'core', 'abis')
        
        if not os.path.exists(abi_dir):
            logger.error(f"ABI directory not found: {abi_dir}")
            raise ConfigError(f"ABI directory not found: {abi_dir}")
            
        for root, dirs, files in os.walk(abi_dir):
            for filename in files:
                if filename.endswith('.json'):
                    file_path = os.path.join(root, filename)
                    # 获取相对于abi_dir的路径
                    relative_path = os.path.relpath(file_path, abi_dir)
                    # 移除.json后缀
                    contract_name = os.path.splitext(relative_path)[0]
                    # 将路径分隔符替换为/
                    contract_name = contract_name.replace(os.path.sep, '/')
                    
                    try:
                        with open(file_path, 'r', encoding='utf-8') as f:
                            self._abis[contract_name] = json.load(f)
                            logger.debug(f"Loaded ABI for {contract_name} from {file_path}")
                    except Exception as e:
                        logger.error(f"Failed to load ABI file {file_path}: {str(e)}")
                        raise ConfigError(f"Failed to load ABI file {file_path}: {str(e)}")
    
    def get_abi(self, contract_name: str) -> list:
        """获取指定合约的ABI"""
        if contract_name not in self._abis:
            raise ConfigError(f"ABI not found for contract: {contract_name}")
        return self._abis[contract_name] 


from typing import Dict, Optional
from ...core.interfaces import IDexProvider
from ...utils.web3_helper import Web3Helper
from ...config.settings import WALLET_CONFIG, NATIVE_TOKENS
from ...utils.logger import get_logger
from .client import OKXClient

logger = get_logger(__name__)

class OKXProvider(IDexProvider):
    """OKX DEX Provider 实现"""
    
    def __init__(self):
        self._client = OKXClient()
        self.wallet_config = WALLET_CONFIG["default"]
    
    @property
    def client(self):
        return self._client
    
    def _get_web3_helper(self, chain_id: str) -> Web3Helper:
        return Web3Helper.get_instance(chain_id)
    
    def _get_amount_in_wei(self, web3_helper: Web3Helper, token_address: str, amount: str) -> str:
        """将代币金额转换为链上精度"""
        try:
            chain_id = web3_helper.chain_id
            if token_address.lower() == "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee":
                if chain_id not in NATIVE_TOKENS:
                    raise ValueError(f"Unsupported chain ID: {chain_id}")
                decimals = NATIVE_TOKENS[chain_id]["decimals"]
            else:
                decimals = web3_helper.get_token_decimals(token_address)
            
            return str(web3_helper.parse_token_amount(amount, decimals))
        except Exception as e:
            logger.error(f"Failed to convert amount {amount} for token {token_address}: {str(e)}")
            raise

    def get_quote(self, chain_id: str, from_token: str,  to_token: str, amount: str, **kwargs) -> Dict:
        """获取报价"""
        try:
            web3_helper = self._get_web3_helper(chain_id)
            raw_amount = self._get_amount_in_wei(web3_helper, from_token, amount)
            
            params = {
                "chainId": chain_id,
                "fromTokenAddress": from_token,
                "toTokenAddress": to_token,
                "amount": raw_amount,
                **kwargs
            }
            
            quote_result = self.client.get_quote(params)
            logger.info(f"Got quote for {amount} from {from_token} to {to_token}")
            return quote_result
            
        except Exception as e:
            logger.error(f"Failed to get quote: {str(e)}")
            raise

    def check_and_approve(self, chain_id: str, token_address: str, 
                         owner_address: str, amount: int) -> Optional[str]:
        """检查并处理代币授权"""
        try:
            web3_helper = self._get_web3_helper(chain_id)
            
            approve_data = self.client.get_approve_transaction({
                "chainId": chain_id,
                "tokenContractAddress": token_address,
                "approveAmount": str(amount)
            })
            
            spender_address = approve_data["data"][0]["dexContractAddress"]
            current_allowance = web3_helper.get_allowance(token_address, owner_address, spender_address)
            
            if current_allowance < amount:
                logger.info(f"Current allowance {current_allowance} is less than required amount {amount}, approving...")
                
                contract = web3_helper.web3.eth.contract(
                    address=web3_helper.web3.to_checksum_address(token_address),
                    abi=web3_helper.abi_helper.get_abi('erc20')
                )
                
                tx_params = {
                    'from': owner_address,
                    'nonce': web3_helper.web3.eth.get_transaction_count(owner_address),
                    'gasPrice': int(approve_data["data"][0]["gasPrice"]),
                    'gas': int(approve_data["data"][0]["gasLimit"]),
                }
                
                approve_tx = contract.functions.approve(
                    web3_helper.web3.to_checksum_address(spender_address),
                    amount
                ).build_transaction(tx_params)
                
                tx_hash = web3_helper.send_transaction(approve_tx, self.wallet_config["private_key"])
                logger.info(f"Approval transaction sent: {tx_hash}")
                return tx_hash
            
            return None
            
        except Exception as e:
            logger.error(f"Failed to check and approve: {str(e)}")
            raise

    def swap(self, chain_id: str, from_token: str, to_token: str, amount: str,
             recipient_address: Optional[str] = None,slippage: str = "0.03", **kwargs) -> str:
        """执行兑换"""
        try:
            web3_helper = self._get_web3_helper(chain_id)
            user_address = self.wallet_config["address"]
            
            raw_amount = self._get_amount_in_wei(web3_helper, from_token, amount)
            
            if from_token.lower() != "0xeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee":
                approve_tx = self.check_and_approve(
                    chain_id=chain_id,
                    token_address=from_token,
                    owner_address=user_address,
                    amount=int(raw_amount)
                )
                
                if approve_tx:
                    web3_helper.web3.eth.wait_for_transaction_receipt(approve_tx)
            
            params = {
                "chainId": chain_id,
                "fromTokenAddress": from_token,
                "toTokenAddress": to_token,
                "amount": raw_amount,
                "userWalletAddress": user_address,
                **kwargs
            }

            if recipient_address:
                params["swapReceiverAddress"] = recipient_address
            
            swap_data = self.client.get_swap(params)
            tx_info = swap_data["data"][0]["tx"]
            
            transaction = {
                "nonce": web3_helper.web3.eth.get_transaction_count(user_address),
                "to": tx_info["to"],
                "gasPrice": int(int(tx_info["gasPrice"]) * 1.5),
                "gas": int(int(tx_info["gas"]) * 1.5),
                "data": tx_info["data"],
                "value": int(tx_info["value"]),
                "chainId": int(chain_id)
            }
            
            tx_hash = web3_helper.send_transaction(transaction, self.wallet_config["private_key"])
            logger.info(f"Swap transaction sent: {tx_hash}")
            return tx_hash
            
        except Exception as e:
            logger.error(f"Failed to execute swap: {str(e)}")
            raise 


class DexAggregatorException(Exception):
    """基础异常类"""
    def __init__(self, message: str, code: int = None):
        self.message = message
        self.code = code
        super().__init__(self.message)

class ProviderError(DexAggregatorException):
    """Provider相关错误"""
    pass

class QuoteError(DexAggregatorException):
    """询价相关错误"""
    pass

class SwapError(DexAggregatorException):
    """兑换相关错误"""
    pass

class ConfigError(DexAggregatorException):
    """配置相关错误"""
    pass
 


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="dex-aggregator",
    version="0.3.1",
    author="hedeqiang",
    author_email="laravel_code@163.com",
    description="A Python client for interacting with DEX aggregators",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hedeqiang/python_dex_aggregator",
    packages=find_packages(),
    package_data={
        "dex_aggregator": ["core/abis/*.json"],
    },
    install_requires=[
        "eth-typing>=5.1.0",
        "python-dotenv>=1.0.1",
        "requests>=2.32.3",
        "web3>=7.8.0",
        "solders>=0.19.0",
        "solana>=0.30.2",
        "base58>=2.1.1",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
) 


import hmac
import base64
import requests
from typing import Dict, Optional
from urllib.parse import urlencode
from datetime import datetime, timezone

from dex_aggregator.core.exceptions import ProviderError
from dex_aggregator.config.settings import API_CONFIG
from dex_aggregator.utils.logger import get_logger

logger = get_logger(__name__)

class OKXClient:
    def __init__(self):
        self.config = API_CONFIG["OKX"]
        self.base_url = "https://www.okx.com"
        self.api_key = self.config["api_key"]
        self.secret_key = self.config["secret_key"]
        self.passphrase = self.config["passphrase"]

    def _get_timestamp(self) -> str:
        """获取ISO格式的UTC时间戳"""
        dt = datetime.now(timezone.utc)
        return dt.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'

    def _generate_signature(self, timestamp: str, method: str, request_path: str) -> str:
        """
        生成签名
        timestamp + method + requestPath + body
        
        注意：对于 GET 请求，body 为空，但 requestPath 需要包含查询参数
        """
        message = f"{timestamp}{method}{request_path}"
        logger.debug(f"Pre-hash String: {message}")
        
        mac = hmac.new(
            bytes(self.secret_key, encoding='utf8'),
            bytes(message, encoding='utf-8'),
            digestmod='sha256'
        )
        signature = base64.b64encode(mac.digest()).decode()
        logger.debug(f"Generated Signature: {signature}")
        return signature

    def _request(self, method: str, endpoint: str, params: Optional[Dict] = None) -> Dict:
        """发送请求"""
        try:
            # 1. 构建完整的 API 路径
            api_path = '/api/v5/dex/aggregator'
            full_endpoint = f"{api_path}{endpoint}"
            
            # 2. 处理查询参数
            query_string = ""
            if params:
                # 确保参数值为字符串类型
                params = {k: str(v) for k, v in params.items() if v is not None}
                # 按键排序参数
                sorted_params = sorted(params.items())
                query_string = "?" + urlencode(sorted_params)
            
            # 3. 构建用于签名的请求路径
            request_path = f"{full_endpoint}{query_string}"
            
            # 4. 生成时间戳和签名
            timestamp = self._get_timestamp()
            signature = self._generate_signature(timestamp, method, request_path)

            # 5. 准备请求头
            headers = {
                "OK-ACCESS-KEY": self.api_key,
                "OK-ACCESS-SIGN": signature,
                "OK-ACCESS-TIMESTAMP": timestamp,
                "OK-ACCESS-PASSPHRASE": self.passphrase,
                "Content-Type": "application/json"
            }

            # 6. 构建完整的 URL
            url = f"{self.base_url}{request_path}"
            
            # 调试信息
            logger.debug(f"Timestamp: {timestamp}")
            logger.debug(f"Method: {method}")
            logger.debug(f"Request Path for Signature: {request_path}")
            logger.debug(f"Request URL: {url}")
            logger.debug(f"Request Headers: {headers}")
            logger.debug(f"Request Params: {params}")

            # 7. 发送请求 - 注意这里不再使用 params 参数，因为已经包含在 URL 中
            response = requests.request(
                method=method,
                url=url,
                headers=headers
            )
            
            logger.debug(f"Response Status: {response.status_code}")
            logger.debug(f"Response Text: {response.text}")
            
            response.raise_for_status()
            return response.json()
            
        except requests.exceptions.RequestException as e:
            logger.error(f"Request failed: {str(e)}")
            raise ProviderError(f"OKX API request failed: {str(e)}")

    def get_supported_chains(self, chain_id: Optional[str] = None) -> Dict:
        """获取支持的链列表"""
        params = {"chainId": chain_id} if chain_id else None
        return self._request("GET", "/supported/chain", params)

    def get_token_list(self, chain_id: str) -> Dict:
        """获取币种列表"""
        params = {"chainId": chain_id}
        return self._request("GET", "/all-tokens", params)

    def get_liquidity(self,chain_id: str) -> Dict:
        """获取流动性"""
        params = {"chainId": chain_id}
        return self._request("GET", "/liquidity", params)

    def get_quote(self, params: Dict) -> Dict:
        """获取询价信息"""
        return self._request("GET", "/quote", params)

    def get_swap(self, params: Dict) -> Dict:
        """获取兑换信息"""
        return self._request("GET", "/swap", params)

    def get_approve_transaction(self, params: Dict) -> Dict:
        """获取授权交易信息"""
        return self._request("GET", "/approve-transaction", params)

    def get_history(self, chain_id: str, tx_hash: str) -> Dict:
        """根据 txhash 查询单链兑换最终交易状态。"""
        params = {"chainId": chain_id, "txHash": tx_hash}
        return self._request("GET", "/history", params)

    def get_swap_instruction(self, params: Dict) -> Dict:
        """获取兑换指令"""
        return self._request("GET", "/swap-instruction", params)